SECRET_KEY = 'test'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test.sqlite',
    }
}

INSTALLED_APPS = [
    'nobot',
]

NOBOT_RECAPTCHA_PRIVATE_KEY = 'privkey'
NOBOT_RECAPTCHA_PUBLIC_KEY = 'pubkey'



import mock
import httpretty
from django.forms import Form
from six.moves.urllib.parse import urlencode

from nobot.client import ReCaptchaClient, HumanCaptchaClient, RecaptchaResponse
from nobot.fields import ReCaptchaField, HumanCaptchaField


class SuccessfulReCaptchaClient(ReCaptchaClient):
    def verify(self, challenge, response, remote_ip):
        return RecaptchaResponse(is_valid=True, error_code=None)


class SuccessfulHumanCaptchaClient(HumanCaptchaClient):
    def verify(self, challenge, response, remote_ip):
        return RecaptchaResponse(is_valid=True, error_code=None)


class ReCaptchaTestForm(Form):
    captcha = ReCaptchaField(
        attrs={'theme': 'white'},
        client_class=SuccessfulReCaptchaClient)


class HumanCaptchaTestForm(Form):
    captcha = HumanCaptchaField(
        attrs={'theme': 'white'},
        client_class=SuccessfulHumanCaptchaClient)


class TestReCaptchaClient(object):
    def setup(self):
        self.verify_data = {
            'privatekey': 'privkey',
            'remoteip': '127.0.0.1',
            'challenge': 'test',
            'response': 'test',
        }

    def test_simple_pass(self):
        form_params = {'recaptcha_response_field': 'test'}
        form = ReCaptchaTestForm(form_params)
        assert form.is_valid()

    @mock.patch('nobot.client.render_to_string')
    def test_render_simple(self, render_to_string, activate_en):
        client = ReCaptchaClient()

        assert not client.nocaptcha

        client.render({})

        render_to_string.assert_called_once_with(
            'captcha/widget.html',
            {
                'api_server': '//www.google.com/recaptcha/api',
                'public_key': 'pubkey',
                'lang': 'en',
                'options': '{"lang": "en"}',
                'challenge_url': '//www.google.com/recaptcha/api/challenge?k=pubkey&hl=en',  # noqa
                'noscript_url': '//www.google.com/recaptcha/api/noscript?k=pubkey&hl=en',  # noqa
            }
        )

    @mock.patch('nobot.client.render_to_string')
    def test_render_has_error(self, render_to_string, activate_en):
        client = ReCaptchaClient()
        client.render({}, 'foo bar')

        render_to_string.assert_called_once_with(
            'captcha/widget.html',
            {
                'api_server': '//www.google.com/recaptcha/api',
                'public_key': 'pubkey',
                'lang': 'en',
                'options': '{"lang": "en"}',
                'challenge_url': '//www.google.com/recaptcha/api/challenge?k=pubkey&hl=en&error=foo+bar',  # noqa
                'noscript_url': '//www.google.com/recaptcha/api/noscript?k=pubkey&hl=en&error=foo+bar',  # noqa
            }
        )

    @mock.patch('nobot.client.render_to_string')
    def test_render_uses_language(self, render_to_string, activate_en):
        client = ReCaptchaClient()
        client.render({'lang': 'de'}, 'foo bar')

        render_to_string.assert_called_once_with(
            'captcha/widget.html',
            {
                'api_server': '//www.google.com/recaptcha/api',
                'public_key': 'pubkey',
                'lang': 'de',
                'options': '{"lang": "de"}',
                'challenge_url': '//www.google.com/recaptcha/api/challenge?k=pubkey&hl=de&error=foo+bar',  # noqa
                'noscript_url': '//www.google.com/recaptcha/api/noscript?k=pubkey&hl=de&error=foo+bar',  # noqa
            }
        )

    @httpretty.activate
    def test_verify_sucess(self):
        args = urlencode(self.verify_data)

        httpretty.register_uri(
            httpretty.GET,
            'https://www.google.com/recaptcha/api/verify?' + args,
            body='true\n\n',
            status=200,
            content_type='plain/text'
        )

        client = ReCaptchaClient()
        response = client.verify('test', 'test', '127.0.0.1')
        last_request = httpretty.last_request()

        assert last_request.path.startswith('/recaptcha/api/verify')
        assert response.is_valid

    @httpretty.activate
    def test_verify_wrong_arguments(self):
        client = ReCaptchaClient()
        response = client.verify('', 'test', '127.0.0.1')

        assert isinstance(
            httpretty.last_request(),
            httpretty.core.HTTPrettyRequestEmpty)

        assert not response.is_valid
        assert response.error_code == 'incorrect-captcha-sol'

    @httpretty.activate
    def test_verify_error(self):
        args = urlencode(self.verify_data)

        httpretty.register_uri(
            httpretty.GET,
            'https://www.google.com/recaptcha/api/verify?' + args,
            body='fail\nerror_code\n',
            status=200,
            content_type='plain/text'
        )

        client = ReCaptchaClient()
        response = client.verify('test', 'test', '127.0.0.1')
        last_request = httpretty.last_request()

        assert last_request.path.startswith('/recaptcha/api/verify')
        assert not response.is_valid
        assert response.error_code == 'error_code'


class TestHumanaptchaClient(object):
    def setup(self):
        self.verify_data = {
            'secret': 'privkey',
            'remoteip': '127.0.0.1',
            'response': 'test',
        }

    def test_simple_pass(self):
        form_params = {'g-recaptcha-response': 'test'}
        form = HumanCaptchaTestForm(form_params)
        assert form.is_valid()

    @mock.patch('nobot.client.render_to_string')
    def test_render_simple(self, render_to_string, activate_en):
        client = HumanCaptchaClient()

        assert client.nocaptcha

        client.render({})

        render_to_string.assert_called_once_with(
            'captcha/widget_nocaptcha.html',
            {
                'api_server': '//www.google.com/recaptcha/api',
                'public_key': 'pubkey',
                'lang': 'en',
                'options': '{"lang": "en"}',
                'challenge_url': '//www.google.com/recaptcha/api/challenge?k=pubkey&hl=en',  # noqa
                'noscript_url': '//www.google.com/recaptcha/api/noscript?k=pubkey&hl=en',  # noqa
            }
        )

    @mock.patch('nobot.client.render_to_string')
    def test_render_has_error(self, render_to_string, activate_en):
        client = HumanCaptchaClient()
        client.render({}, 'foo bar')

        render_to_string.assert_called_once_with(
            'captcha/widget_nocaptcha.html',
            {
                'api_server': '//www.google.com/recaptcha/api',
                'public_key': 'pubkey',
                'lang': 'en',
                'options': '{"lang": "en"}',
                'challenge_url': '//www.google.com/recaptcha/api/challenge?k=pubkey&hl=en&error=foo+bar',  # noqa
                'noscript_url': '//www.google.com/recaptcha/api/noscript?k=pubkey&hl=en&error=foo+bar',  # noqa
            }
        )

    @mock.patch('nobot.client.render_to_string')
    def test_render_uses_language(self, render_to_string, activate_en):
        client = HumanCaptchaClient()
        client.render({'lang': 'de'}, 'foo bar')

        render_to_string.assert_called_once_with(
            'captcha/widget_nocaptcha.html',
            {
                'api_server': '//www.google.com/recaptcha/api',
                'public_key': 'pubkey',
                'lang': 'de',
                'options': '{"lang": "de"}',
                'challenge_url': '//www.google.com/recaptcha/api/challenge?k=pubkey&hl=de&error=foo+bar',  # noqa
                'noscript_url': '//www.google.com/recaptcha/api/noscript?k=pubkey&hl=de&error=foo+bar',  # noqa
            }
        )

    @httpretty.activate
    def test_verify_sucess(self):
        args = urlencode(self.verify_data)

        httpretty.register_uri(
            httpretty.GET,
            'https://www.google.com/recaptcha/api/siteverify?' + args,
            body='{"error-codes": [], "success": true}',
            status=200,
            content_type='plain/text'
        )

        client = HumanCaptchaClient()
        response = client.verify('test', 'test', '127.0.0.1')
        last_request = httpretty.last_request()

        assert last_request.path.startswith('/recaptcha/api/siteverify')
        assert response.is_valid

    @httpretty.activate
    def test_verify_error(self):
        args = urlencode(self.verify_data)

        httpretty.register_uri(
            httpretty.GET,
            'https://www.google.com/recaptcha/api/siteverify?' + args,
            body='{"error-codes": [], "success": false}',
            # Google returns 200 even in case of error :-/
            status=200,
            content_type='plain/text'
        )

        client = HumanCaptchaClient()
        response = client.verify('test', 'test', '127.0.0.1')
        last_request = httpretty.last_request()

        assert last_request.path.startswith('/recaptcha/api/siteverify')
        assert not response.is_valid
        assert response.error_code is None



from django import forms
from django.utils.safestring import mark_safe

from .client import ReCaptchaClient, HumanCaptchaClient


class ReCaptchaWidget(forms.widgets.Widget):
    client_class = ReCaptchaClient

    def __init__(self, attrs=None, *args, **kwargs):
        self.js_attrs = {} if attrs is None else attrs
        self.client = self.client_class()

        # TODO: move to client?
        if self.client.nocaptcha:
            self.challenge_field = 'g-recaptcha-response'
            self.response_field = 'g-recaptcha-response'
        else:
            self.challenge_field = 'recaptcha_challenge_field'
            self.response_field = 'recaptcha_response_field'

        super(ReCaptchaWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        return mark_safe(u'%s' % self.client.render(self.js_attrs))

    def value_from_datadict(self, data, files, name):
        return [
            data.get(self.challenge_field, None),
            data.get(self.response_field, None)
        ]


class HumanCaptchaWidget(ReCaptchaWidget):
    client_class = HumanCaptchaClient



import sys
import os
import codecs
from setuptools import setup, find_packages


version = '0.5'


if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    os.system('python setup.py bdist_wheel upload')
    print('You probably want to also tag the version now:')
    print('  git tag -a %s -m "version %s"' % (version, version))
    print('  git push --tags')
    sys.exit()


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


test_requires = [
    'pytest>=3.1.2',
    'pytest-cov>=2.5.1',
    'pytest-flakes>=2.0.0',
    'pytest-pep8>=1.0.6',
    'pytest-django>=3.1.2',
    'pep8==1.7.0',
    'httpretty>=0.8.14,<0.9',
    'mock>=2.0.0',
]


install_requires = [
    'Django>=1.8,<2.0',
    'requests>=1.1.0',
    'six>=1.9.0',
]


setup(
    name='nobot',
    version=version,
    description='Django recaptcha form field/widget app.',
    long_description=read('README.rst') + read('AUTHORS.rst'),
    author='Christopher Grebs',
    author_email='cg@webshox.org',
    install_requires=install_requires,
    extras_require={
        'tests': test_requires,
    },
    license='BSD',
    url='https://github.com/EnTeQuak/nobot',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Programming Language :: Python',
        'License :: OSI Approved :: BSD License',
        'Development Status :: 5 - Production/Stable',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: CPython',
        'Framework :: Django',
    ],
    zip_safe=False,
)



import sys
import socket

from django import forms
from django.utils.encoding import smart_text

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from .client import ReCaptchaClient, HumanCaptchaClient
from .widgets import ReCaptchaWidget, HumanCaptchaWidget


class ReCaptchaField(forms.CharField):
    default_error_messages = {
        'captcha_invalid': _('Incorrect, please try again.'),
        'captcha_error': _('Error verifying input, please try again.'),
    }

    default_attrs = {}

    widget_class = ReCaptchaWidget
    client_class = ReCaptchaClient

    def __init__(self, public_key=None, private_key=None, attrs=None,
                 *args, **kwargs):
        """
        ReCaptchaField can accepts attributes which is a dictionary of
        attributes to be passed to the ReCaptcha widget class. The widget will
        loop over any options added and create the RecaptchaOptions
        JavaScript variables as specified in
        https://code.google.com/apis/recaptcha/docs/customization.html
        """
        copied_attrs = self.default_attrs.copy()

        if attrs is not None:
            copied_attrs.update(attrs)

        self.widget = self.widget_class(attrs=copied_attrs)
        self.client = kwargs.pop('client_class', self.client_class)()
        self.required = True
        super(ReCaptchaField, self).__init__(*args, **kwargs)

    def get_remote_ip(self):
        # TODO: get rid of this crap!
        f = sys._getframe()
        while f:
            if 'request' in f.f_locals:
                request = f.f_locals['request']
                if request:
                    remote_ip = request.META.get('REMOTE_ADDR', '')
                    forwarded_ip = request.META.get('HTTP_X_FORWARDED_FOR', '')
                    ip = remote_ip if not forwarded_ip else forwarded_ip
                    return ip
            f = f.f_back

    def clean(self, values):
        super(ReCaptchaField, self).clean(values[1])

        challenge_value = smart_text(values[0])
        response_value = smart_text(values[1])

        try:
            response = self.client.verify(
                challenge_value,
                response_value,
                remote_ip=self.get_remote_ip())
        except socket.error:
            # Catch timeouts etc.
            raise ValidationError(self.error_messages['captcha_error'])

        if not response.is_valid:
            raise ValidationError(self.error_messages['captcha_invalid'])

        return values[0]


class HumanCaptchaField(ReCaptchaField):
    widget_class = HumanCaptchaWidget
    client_class = HumanCaptchaClient



import json
import collections

import requests
from django.conf import settings
from django.template.loader import render_to_string
from django.utils.encoding import force_bytes, force_text
from django.utils.safestring import mark_safe
from django.utils.translation import get_language

from six.moves import urllib


RecaptchaResponse = collections.namedtuple(
    'RecaptchaResponse',
    'is_valid, error_code')


class ReCaptchaClient(object):
    VERIFY_URL = 'https://www.google.com/recaptcha/api/verify'
    API_SERVER = '//www.google.com/recaptcha/api'
    SUPPORTED_LANGUAGES = ('en', 'nl', 'fr', 'de', 'pt', 'ru', 'es', 'tr')

    site_key = None
    secret_key = None
    nocaptcha = False
    template = 'captcha/widget.html'

    def __init__(self, site_key=None, secret_key=None):
        self.site_key = site_key or settings.NOBOT_RECAPTCHA_PUBLIC_KEY
        self.secret_key = secret_key or settings.NOBOT_RECAPTCHA_PRIVATE_KEY

    def render(self, attrs, error=None):
        options = attrs.copy()

        if 'lang' not in options:
            options['lang'] = get_language()[:2]

        args = collections.OrderedDict((
            ('k', self.site_key),
            ('hl', options['lang'])
        ))

        if error:
            args['error'] = error

        challenge_url = (
            self.API_SERVER + '/challenge?' + urllib.parse.urlencode(args))
        noscript_url = (
            self.API_SERVER + '/noscript?' + urllib.parse.urlencode(args))

        return render_to_string(
            self.template,
            {
                'api_server': self.API_SERVER,
                'public_key': self.site_key,
                'lang': options['lang'],
                'options': mark_safe(json.dumps(options)),
                'challenge_url': challenge_url,
                'noscript_url': noscript_url
            }
        )

    def verify(self, challenge, response, remote_ip):
        if not (response and challenge and len(response) and len(challenge)):
            return RecaptchaResponse(
                is_valid=False,
                error_code='incorrect-captcha-sol'
            )

        data = {
            'privatekey': force_bytes(self.secret_key),
            'remoteip': force_bytes(remote_ip),
            'challenge': force_bytes(challenge),
            'response': force_bytes(response),
        }

        r = requests.get(self.VERIFY_URL, params=data)

        return_code, error_code = force_text(r.content).splitlines()

        if (str(return_code) == 'true'):
            return RecaptchaResponse(is_valid=True, error_code=None)
        else:
            return RecaptchaResponse(is_valid=False, error_code=error_code)


class HumanCaptchaClient(ReCaptchaClient):
    VERIFY_URL = 'https://www.google.com/recaptcha/api/siteverify'
    template = 'captcha/widget_nocaptcha.html'
    nocaptcha = True

    def verify(self, challenge, response, remote_ip):
        data = {
            'secret': force_bytes(self.secret_key),
            'response': force_bytes(response),
            'remoteip': force_bytes(remote_ip)
        }

        verified = requests.get(self.VERIFY_URL, params=data)

        if verified.status_code == 200:
            data = verified.json()
            if data['success']:
                return RecaptchaResponse(is_valid=True, error_code=None)

        return RecaptchaResponse(is_valid=False, error_code=None)
